//  *****************************************************************************************************************
//  *                                         SpikenzieLabs.com                                                     *
//  *                                           Calculator Kit                                                      *
//  *****************************************************************************************************************
// BY: MARK DEMERS - SpikenzieLabs
// Dec 2012
// VERSION 1.1
//
// Brief:
// Sketch used in the SpikenzieLabs Calculator Kit, more info and build instructions at http://www.spikenzielabs.com/calc
// 
//
// LEGAL:
// This code is provided as is. No guaranties or warranties are given in any form. It is your responsibilty to 
// determine this codes suitability for your application.

// The SpikenzieLabs Calculator Kit software and hardware is licensed under Attribution-NonCommercial.  If you would like to 
// make a commercial product based on our works in whole or in part, please contact liscencing@spikenzielabs.com to discuss 
// possible licensing arrangements.
//
// NOTICE:
// SpikenzieLabs does not permit use of this device or code to calculate spacecraft trajectories, other mission critcal
// calculations or life support uses. This is a novelty calculator and even thou every attempt has been made it
// calculate accurately it's processor is limited to a maximum of 6 or 7 digits of "reasonable" accuracy.
// 
// Changes:
// v0  - Float numbers test - for future calculator Kit
// v1c - Started to add decimal point implementation
// v1E - Changed INT to long formate (needed larger numbers)
// v1f - Tring to reduce variable sizes (have some strange results - may be too much SRAM memory usage)
//     - ie: Changing int variable to byte
// v1f2 - Removed Long interger math
// v1f3 - Removed most local variable (reusing globals in most routines)
// v1f4 - reversed declaration order floats 1st boolean last (fixed a lot of "stupid this should be working bugs")
// v1g  - Reworking debounce
// v1h  - Better power saving by pin manipulation during sleep
//      - Miss. bugs with repeated functions (when user presses = again or =-/* then addes another #
// v1h2 - Removed display variable - disp[] write in number decode routine
//      - Separated floats into long and decimal value to help with long decimal rounding errors
// v1i  - fixed decimal place bugs
//      - Added WDT (Watch Dog timer) overflow at 2 seconds reset (reset for "crazy" key mashing errors)
// v1J  - Frustration: Simple operation works - mashing ====== causes - only sometimes an error
// v1k  - Found bug with clearing keys array out of range 
// v1.1 - Release
// v1.2 - Bobricius upgrade ... talking and enahanced to 8 digits
// -----------------------------------------------------------------------------------------------------------------
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include <MemoryFree.h>
#include <TimerOne.h>
#include <avr/pgmspace.h>
// -----------------------------------------------------------------------------------------------------------------
uint8_t const spONE[]       PROGMEM = {
  0xA6,0x90,0x76,0xCC,0x43,0x6B,0xB9,0x42,0xD8,0x35,0x77,0xA9,0x19,0x72,0xE5,0x46,0xD2,0x62,0x56,0xCB,0xAC,0x9F,0x68,0xE1,0x4E,0x23,0x09,0x71,0x23,0x55,0x3A,0x9E,0xB8,0xC4,0x4B,0x77,0x6D,0x74,0xE3,0x93,0x54,0x7B,0xE5,0x6E,0x2D,0x4F,0xD6,0xEC,0x56,0x84,0xB5,0x3A,0x45,0x0D,0x5D,0xA1,0xD6,0xA8,0xF4,0xC5,0x58,0x84,0x78,0xEC,0x34,0x64,0x6D,0x16,0xEA,0x8D,0xC2,0x90,0x95,0x66,0xB1,0x25,0x0E,0x43,0x92,0x1A,0x43,0x9E,0x26,0x0C,0x49,0x49,0xB4,0x78,0xEA,0x34,0x16,0x1B,0xA6,0xEA,0xB1,0xC9,0x24,0x82,0xBB,0x88,0xC8,0x06,0x00,0xF8,0x26};
uint8_t const spTWO[]       PROGMEM = {
  0x0E,0x58,0x56,0xCB,0x00,0xD7,0xAB,0x19,0xE0,0x38,0xCB,0xE6,0x0C,0x65,0x8A,0x61,0xAD,0x86,0x9D,0xA5,0x79,0xA4,0x2D,0x1E,0x6E,0x61,0x69,0x9A,0xF6,0x79,0x04,0x85,0xA4,0x6B,0xC8,0xEF,0x16,0x17,0x94,0xA1,0xC1,0xBB,0x5B,0x5E,0x61,0x86,0x05,0x7E,0x0A,0xA5,0x0A,0x25,0xA6,0x75,0x3A,0x34,0x8E,0x5F,0xA8,0xEB,0x4B,0xD7,0x49,0xBB,0xE6,0x12,0x8F,0xDD,0xE0,0xC5,0x78,0xB2,0xBC,0x36,0x83,0x15,0xEB,0x2E,0xF2,0x48,0x0D,0x46,0x9E,0x89,0xE9,0x2B,0x31,0xC8,0x32,0x2C,0x9E,0xAB,0xFF,0x5A,0x26};
uint8_t const spTHREE[]     PROGMEM = {
  0x08,0xA8,0xC2,0x8C,0x02,0x04,0x68,0x2A,0xDC,0xF9,0x51,0x5B,0x96,0x79,0x8D,0x10,0xE5,0xCA,0x2E,0x9A,0x76,0x72,0xD0,0xC2,0x5C,0x25,0x21,0x23,0xCD,0x0C,0x4F,0xD4,0x22,0x7A,0x46,0x34,0x3E,0xF1,0x48,0x86,0xD2,0xB1,0xEA,0x24,0x33,0x16,0x62,0xE7,0xAA,0x55,0xAC,0xD4,0x04,0xD5,0x8D,0x47,0xB3,0x53,0x33,0xE4,0x2C,0x69,0xED,0x4E,0x81,0x30,0x53,0xA7,0xF5,0xBB,0x14,0x43,0xF4,0x92,0x36,0xEC,0x92,0x04,0xD5,0x4B,0xD2,0xB8,0xAB,0x23,0xF4,0x34,0xCE,0x63,0x19,0x57,0x73,0x84,0xAE,0x26,0x69,0x9C,0x8D,0xC0,0xAB,0x6B,0x87,0xB1,0x7B,0x94,0x99,0x8A,0xF2,0x5A,0x66};
uint8_t const spFOUR[]      PROGMEM = {
  0x08,0xA8,0xCA,0x94,0x00,0x5D,0x84,0x31,0x20,0xA9,0x0A,0x05,0x54,0x1D,0x16,0xBC,0xAA,0xDC,0xC3,0xC4,0xCE,0x0A,0x92,0xDA,0x94,0xA0,0x4D,0xC7,0x4F,0x7C,0xDA,0x0B,0x73,0x9F,0x20,0xAA,0x2D,0x73,0xCE,0x75,0x92,0x28,0xB7,0x23,0xD9,0xF7,0xCA,0x82,0xB9,0x16,0xD3,0x5C,0xA3,0x88,0x7A,0x2A,0x83,0x72,0x8D,0x2E,0xEA,0xED,0x08,0xCC,0xD3,0x86,0x68,0xB6,0xCD,0x29,0x77,0x19,0xA3,0xDE,0xD6,0xA0,0xDE,0x65,0x0C,0x66,0x5B,0x82,0x73,0x97,0x31,0xD8,0x2D,0x0B,0xEA,0x53,0xC6,0x10,0xB6,0x24,0xB8,0x77,0x99,0x42,0x98,0x4C,0xE5,0xDC,0x6E,0x0A,0xA3,0x4C,0xC3,0x17,0xFE,0x5A,0x16};
uint8_t const spFIVE[]      PROGMEM = {
  0x08,0xE8,0xD2,0x15,0x03,0x02,0xA8,0x3A,0x5D,0x01,0x55,0x96,0x0E,0x3F,0xA5,0x70,0xE7,0x98,0xBD,0x82,0xDC,0xDB,0x5D,0xAD,0xD6,0x0A,0x4A,0xF8,0x70,0xF5,0x86,0x27,0x2E,0xFE,0xD3,0xC5,0x6B,0x9D,0xB4,0xE9,0xAB,0x94,0x68,0x78,0xAA,0xEA,0x2F,0xC3,0xA3,0xF1,0xE9,0x6A,0xD8,0x88,0xF0,0xD9,0xA7,0x6F,0x61,0xA2,0xDC,0x66,0xAF,0xBE,0x85,0xB6,0x34,0x9B,0xB5,0xFA,0xEE,0x5D,0xD2,0x73,0xCE,0xEA,0xBB,0x33,0x8A,0xAC,0x3A,0x63,0xE8,0x56,0x29,0xAB,0xAB,0x94,0xBE,0x67,0x45,0xCB,0x9E,0x5D,0xFA,0xE6,0x04,0xB3,0x72,0x76,0xEA,0x8A,0x26,0x6F,0xCF,0x29,0xA9,0x4D,0xCA,0x34,0x3D,0x6A,0xC7,0xC1,0x0F,0xFF,0x5A,0x56};
uint8_t const spSIX[]       PROGMEM = {
  0x04,0xF8,0x82,0x8D,0x03,0x0A,0xF8,0xCA,0xDC,0x01,0x5F,0x79,0x28,0xE0,0xEB,0x30,0x05,0x7C,0xED,0xBE,0xBC,0xAD,0x8A,0x34,0x62,0xD5,0x0D,0x4E,0xB4,0xDC,0x12,0x87,0x77,0x1A,0xD5,0x28,0x09,0x1E,0xBE,0x24,0x36,0xA8,0x15,0xCA,0xDD,0x45,0x64,0x03,0x80,0x01,0xF2,0x54,0x31,0xC0,0x57,0x15,0x02,0xF8,0xCA,0x8C,0x01,0xDF,0x58,0x12,0xE0,0x1B,0x0B,0x04,0x7C,0x2F,0x86,0x80,0xEF,0xD4,0xFE,0x5A,0x36};
uint8_t const spSEVEN[]     PROGMEM = {
  0x08,0xF8,0x5C,0x9D,0x01,0x5F,0x78,0x08,0xE0,0x8B,0x74,0x05,0x7C,0x15,0xAE,0x81,0x61,0x57,0x6B,0x9A,0xDC,0x75,0x8E,0xD7,0x6D,0x48,0x89,0x2F,0x3E,0x41,0x0B,0xED,0x29,0x3A,0xFB,0x44,0xD5,0xA7,0x87,0xDA,0xA2,0x96,0xC4,0x50,0x11,0xCA,0xB5,0x54,0xAD,0x65,0x6A,0x88,0xB8,0x6A,0x6D,0x0E,0x63,0xC6,0xD6,0xB8,0x8D,0xB9,0x46,0x38,0xD9,0xEC,0x34,0xE5,0xAC,0x45,0xBE,0x75,0xC2,0x94,0x84,0x78,0xBB,0x37,0xB6,0x93,0x9E,0xC8,0x24,0x82,0xBB,0x88,0xC8,0x06,0x00,0x80,0xFF,0x5A,0x76};
uint8_t const spEIGHT[]     PROGMEM = {
  0x23,0x1E,0xC5,0x58,0x33,0xA7,0x9E,0xA0,0x6A,0xF1,0xAD,0x9E,0xB2,0xE2,0xEE,0x49,0xAB,0x3A,0xCA,0x2A,0x66,0x72,0x94,0xE9,0xDA,0xBB,0x0A,0xC3,0x30,0x8C,0xB5,0x1D,0x5B,0x4C,0x42,0xB9,0xBB,0x88,0x6C,0x00,0x00,0x01,0x5C,0x91,0x26,0x01,0x04,0x14,0x9F,0xFA,0x5A,0x0E};
uint8_t const spNINE[]      PROGMEM = {
  0xA1,0x4A,0x4C,0xF4,0x31,0xDD,0x85,0x32,0x71,0xB6,0xC7,0x74,0x97,0x8A,0xCC,0x54,0x1F,0xCB,0x5D,0xC9,0x33,0x35,0x7D,0xCC,0xE4,0x2D,0x6B,0x20,0xF9,0x58,0xEB,0x9F,0xA4,0xCB,0x56,0x13,0x6F,0x74,0xE2,0xAE,0x37,0x5C,0x3D,0xD1,0x89,0x9B,0xBD,0x74,0xF1,0x44,0x27,0x69,0xE6,0xDA,0xD5,0x13,0x9D,0xB4,0xAB,0x9B,0x10,0x4F,0x74,0xF2,0x29,0x67,0xC2,0x3C,0xE5,0x29,0xA7,0xE9,0xAA,0x90,0x54,0xA7,0x9A,0xBE,0x3D,0x52,0x5A,0x9D,0x66,0xC5,0x51,0x49,0x6B,0x74,0xDA,0x95,0x46,0x30,0xA2,0xD1,0xE8,0x66,0x2E,0xE4,0xCA,0xCA,0x6D,0x58,0x21,0x89,0x3A,0x23,0x87,0x21,0x73,0xB5,0x71,0x4D,0x6A,0x86,0x20,0x2C,0xCE,0xCD,0x89,0x1A,0x82,0xB0,0x5C,0xB5,0x24,0x6A,0x08,0x22,0x2A,0x2D,0x3D,0xAB,0x3E,0x4A,0xAD,0x34,0xF3,0xF5,0x5A,0x4E};

uint8_t const spONE2[]       PROGMEM = {
  0xCC,0x67,0x75,0x42,0x59,0x5D,0x3A,0x4F,0x9D,0x36,0x63,0xB7,0x59,0xDC,0x30,0x5B,0x5C,0x23,0x61,0xF3,0xE2,0x1C,0xF1,0xF0,0x98,0xC3,0x4B,0x7D,0x39,0xCA,0x1D,0x2C,0x2F,0xB7,0x15,0xEF,0x70,0x79,0xBC,0xD2,0x46,0x7C,0x52,0xE5,0xF1,0x4A,0x6A,0xB3,0x71,0x47,0xC3,0x2D,0x39,0x34,0x4B,0x23,0x35,0xB7,0x7A,0x55,0x33,0x8F,0x59,0xDC,0xA2,0x44,0xB5,0xBC,0x66,0x72,0x8B,0x64,0xF5,0xF6,0x98,0xC1,0x4D,0x42,0xD4,0x27,0x62,0x38,0x2F,0x4A,0xB6,0x9C,0x88,0x68,0xBC,0xA6,0x95,0xF8,0x5C,0xA1,0x09,0x86,0x77,0x91,0x11,0x5B,0xFF,0x0F};
uint8_t const spTWO2[]       PROGMEM = {
  0x0E,0x38,0x6E,0x25,0x00,0xA3,0x0D,0x3A,0xA0,0x37,0xC5,0xA0,0x05,0x9E,0x56,0x35,0x86,0xAA,0x5E,0x8C,0xA4,0x82,0xB2,0xD7,0x74,0x31,0x22,0x69,0xAD,0x1C,0xD3,0xC1,0xD0,0xFA,0x28,0x2B,0x2D,0x47,0xC3,0x1B,0xC2,0xC4,0xAE,0xC6,0xCD,0x9C,0x48,0x53,0x9A,0xFF,0x0F};
uint8_t const spTHREE2[]     PROGMEM = {
  0x02,0xD8,0x2E,0x9C,0x01,0xDB,0xA6,0x33,0x60,0xFB,0x30,0x01,0xEC,0x20,0x12,0x8C,0xE4,0xD8,0xCA,0x32,0x96,0x73,0x63,0x41,0x39,0x89,0x98,0xC1,0x4D,0x0D,0xED,0xB0,0x2A,0x05,0x37,0x0F,0xB4,0xA5,0xAE,0x5C,0xDC,0x36,0xD0,0x83,0x2F,0x4A,0x71,0x7B,0x03,0xF7,0x38,0x59,0xCD,0xED,0x1E,0xB4,0x6B,0x14,0x35,0xB7,0x6B,0x94,0x99,0x91,0xD5,0xDC,0x26,0x48,0x77,0x4B,0x66,0x71,0x1B,0x21,0xDB,0x2D,0x8A,0xC9,0x6D,0x88,0xFC,0x26,0x28,0x3A,0xB7,0x21,0xF4,0x1F,0xA3,0x65,0xBC,0x02,0x38,0xBB,0x3D,0x8E,0xF0,0x2B,0xE2,0x08,0xB7,0x34,0xFF,0x0F};
uint8_t const spFOUR2[]      PROGMEM = {
  0x0C,0x18,0xB6,0x9A,0x01,0xC3,0x75,0x09,0x60,0xD8,0x0E,0x09,0x30,0xA0,0x9B,0xB6,0xA0,0xBB,0xB0,0xAA,0x16,0x4E,0x82,0xEB,0xEA,0xA9,0xFA,0x59,0x49,0x9E,0x59,0x23,0x9A,0x27,0x3B,0x78,0x66,0xAE,0x4A,0x9C,0x9C,0xE0,0x99,0xD3,0x2A,0xBD,0x72,0x92,0xEF,0xE6,0x88,0xE4,0x45,0x4D,0x7E,0x98,0x2D,0x62,0x67,0x37,0xF9,0xA1,0x37,0xA7,0x6C,0x94,0xE4,0xC7,0x1E,0xDC,0x3C,0xA5,0x83,0x1F,0x8B,0xEB,0x52,0x0E,0x0E,0x7E,0x2E,0x4E,0xC7,0x31,0xD2,0x79,0xA5,0x3A,0x0D,0xD9,0xC4,0xFF,0x07};
uint8_t const spFIVE2[]      PROGMEM = {
  0x02,0xE8,0x3E,0x8C,0x01,0xDD,0x65,0x08,0x60,0x98,0x4C,0x06,0x34,0x93,0xCE,0x80,0xE6,0xDA,0x9A,0x14,0x6B,0xAA,0x47,0xD1,0x5E,0x56,0xAA,0x6D,0x56,0xCD,0x78,0xD9,0xA9,0x1C,0x67,0x05,0x83,0xE1,0xA4,0xBA,0x38,0xEE,0x16,0x86,0x9B,0xFA,0x60,0x87,0x5B,0x18,0x6E,0xEE,0x8B,0x1D,0x6E,0x61,0xB9,0x69,0x36,0x65,0xBA,0x8D,0xE5,0xE5,0x3E,0x1C,0xE9,0x0E,0x96,0x9B,0x5B,0xAB,0x95,0x2B,0x58,0x6E,0xCE,0xE5,0x3A,0x6A,0xF3,0xB8,0x35,0x84,0x7B,0x05,0xA3,0xE3,0x36,0xEF,0x92,0x19,0xB4,0x86,0xDB,0xB4,0x69,0xB4,0xD1,0x2A,0x4E,0x65,0x9A,0x99,0xCE,0x28,0xD9,0x85,0x71,0x4C,0x18,0x6D,0x67,0x47,0xC6,0x5E,0x53,0x4A,0x9C,0xB5,0xE2,0x85,0x45,0x26,0xFE,0x7F};
uint8_t const spSIX2[]       PROGMEM = {
  0x0E,0xD8,0xAE,0xDD,0x03,0x0E,0x38,0xA6,0xD2,0x01,0xD3,0xB4,0x2C,0xAD,0x6A,0x35,0x9D,0xB1,0x7D,0xDC,0xEE,0xC4,0x65,0xD7,0xF1,0x72,0x47,0x24,0xB3,0x19,0xD9,0xD9,0x05,0x70,0x40,0x49,0xEA,0x02,0x98,0xBE,0x42,0x01,0xDF,0xA4,0x69,0x40,0x00,0xDF,0x95,0xFC,0x3F};
uint8_t const spSEVEN2[]     PROGMEM = {
  0x02,0xB8,0x3A,0x8C,0x01,0xDF,0xA4,0x73,0x40,0x01,0x47,0xB9,0x2F,0x33,0x3B,0x73,0x5F,0x53,0x7C,0xEC,0x9A,0xC5,0x63,0xD5,0xD1,0x75,0xAE,0x5B,0xFC,0x64,0x5C,0x35,0x87,0x91,0xF1,0x83,0x36,0xB5,0x68,0x55,0xC5,0x6F,0xDA,0x45,0x2D,0x1C,0x2D,0xB7,0x38,0x37,0x9F,0x60,0x3C,0xBC,0x9A,0x85,0xA3,0x25,0x66,0xF7,0x8A,0x57,0x1C,0xA9,0x67,0x56,0xCA,0x5E,0xF0,0xB2,0x16,0xB2,0xF1,0x89,0xCE,0x8B,0x92,0x25,0xC7,0x2B,0x33,0xCF,0x48,0xB1,0x99,0xB4,0xF3,0xFF};
uint8_t const spEIGHT2[]     PROGMEM = {
  0xC3,0x6C,0x86,0xB3,0x27,0x6D,0x0F,0xA7,0x48,0x99,0x4E,0x55,0x3C,0xBC,0x22,0x65,0x36,0x4D,0xD1,0xF0,0x32,0xD3,0xBE,0x34,0xDA,0xC3,0xEB,0x82,0xE2,0xDA,0x65,0x35,0xAF,0x31,0xF2,0x6B,0x97,0x95,0xBC,0x86,0xD8,0x6F,0x82,0xA6,0x73,0x0B,0xC6,0x9E,0x72,0x99,0xCC,0xCB,0x02,0xAD,0x3C,0x9A,0x10,0x60,0xAB,0x62,0x05,0x2C,0x37,0x84,0x00,0xA9,0x73,0x00,0x00,0xFE,0x1F};
uint8_t const spNINE2[]      PROGMEM = {
  0xCC,0xA1,0x26,0xBB,0x83,0x93,0x18,0xCF,0x4A,0xAD,0x2E,0x31,0xED,0x3C,0xA7,0x24,0x26,0xC3,0x54,0xF1,0x92,0x64,0x8B,0x8A,0x98,0xCB,0x2B,0x2E,0x34,0x53,0x2D,0x0E,0x2F,0x57,0xB3,0x0C,0x0D,0x3C,0xBC,0x3C,0x4C,0x4B,0xCA,0xF4,0xF0,0x72,0x0F,0x6E,0x49,0x53,0xCD,0xCB,0x53,0x2D,0x35,0x4D,0x0F,0x2F,0x0F,0xD7,0x0C,0x0D,0x3D,0xBC,0xDC,0x4D,0xD3,0xDD,0xC2,0xF0,0x72,0x52,0x4F,0x57,0x9B,0xC3,0xAB,0x89,0xBD,0x42,0x2D,0x0F,0xAF,0x5A,0xD1,0x71,0x91,0x55,0xBC,0x2C,0xC5,0x3B,0xD8,0x65,0xF2,0x82,0x94,0x18,0x4E,0x3B,0xC1,0x73,0x42,0x32,0x33,0x15,0x45,0x4F,0x79,0x52,0x6A,0x55,0xA6,0xA3,0xFF,0x07};



uint8_t const spZERO[]      PROGMEM = {
  0xA2,0x2B,0x20,0x2D,0xAA,0x1A,0xAA,0xB6,0x81,0xD4,0x88,0x72,0xAC,0x80,0x8F,0xC4,0x34,0x90,0xAA,0x8F,0x44,0xD5,0x43,0x4D,0xCB,0x16,0x33,0x8C,0xB6,0x44,0x23,0x9A,0xDC,0x21,0xA7,0x6B,0xAD,0x60,0x28,0x83,0x9C,0x9E,0x75,0x82,0x69,0x12,0x74,0xB2,0xD1,0xF1,0x87,0x74,0xAA,0x89,0x46,0x27,0x18,0x36,0x39,0x5C,0x17,0x9D,0x60,0xA8,0x92,0x34,0x5B,0xB4,0xA2,0xA6,0xCA,0xD2,0x7D,0xD1,0x88,0x73,0xB6,0xF2,0xA8,0x96,0x23,0xCD,0x83,0x4D,0x23,0x5D,0xB7,0x2C,0x0F,0x74,0xF3,0xC8,0xDC,0x8A,0x5C,0xA9,0xA4,0x3B,0x55,0xAB,0x73,0xE5,0xE2,0x3A,0xD7,0x6D,0x2C,0x49,0x8B,0x3D,0x6F,0x95,0xB1,0x78,0x4D,0xD7,0x78,0x94,0xA6,0x64,0xC3,0x33,0xE2,0x52,0x9A,0x92,0x8D,0x08,0xF3,0x4B,0x61,0x4A,0x31,0xDC,0xB8,0x26,0xBB,0x25,0x05,0x77,0x95,0x7A,0xF4,0x06,0x06};
uint8_t const spPLUS[] PROGMEM = {0x0A,0xC8,0x2A,0x54,0x01,0xCD,0xA1,0xA6,0x22,0x09,0x1B,0x57,0x57,0x35,0xF2,0x2C,0x7C,0x93,0x35,0xF3,0x2A,0x2B,0xD9,0x51,0xD3,0x2D,0xAB,0xCC,0xFC,0x4B,0xC3,0x37,0xAF,0x32,0xC9,0xB7,0x72,0x5F,0x39,0xCA,0xE8,0xD3,0x9B,0x7B,0x72,0xA9,0xBE,0xE4,0x48,0x32,0x27,0x06,0x78,0xC6,0x55,0x01,0x3F,0xA5,0x28,0xE0,0xB7,0x52,0x0E,0x30,0xE0,0xB7,0xD4,0xFF,0x07};
uint8_t const spMINUS[] PROGMEM = {0x6A,0xCE,0x8C,0xF4,0xD4,0x93,0x86,0xA5,0x49,0xE2,0x31,0x73,0x1B,0x96,0x26,0x48,0xC6,0x25,0x59,0x9A,0x1B,0x67,0xEC,0xD4,0x34,0x69,0xEA,0x16,0x31,0x33,0x92,0x9C,0xA1,0x88,0xEC,0x08,0xEA,0x72,0xBA,0x2A,0xA6,0x2D,0xB4,0xCB,0xC9,0xBA,0x19,0xB5,0xD4,0x35,0x27,0xEB,0xA9,0x29,0xCC,0x67,0x9F,0xAC,0x65,0xE7,0xE0,0x9E,0x32,0xAA,0x11,0xD1,0xCD,0xC3,0x4D,0x6B,0x7B,0xA0,0x54,0xF5,0x4C,0x67,0x68,0xA1,0xC4,0xC4,0x17,0x8F,0xA5,0x46,0x37,0x97,0x5E,0xD2,0xAE,0x12,0xDD,0x5D,0x7A,0x89,0xBB,0x72,0x72,0xF5,0xD8,0xD9,0x02,0xA8,0xB2,0x58,0x01,0xDF,0x9B,0x09,0xE0,0xB7,0x74,0x01,0xFC,0xE6,0xA9,0x80,0x9F,0x52,0x05,0xF0,0x67,0xFB,0xFF,0x03};
uint8_t const spEQUALS[] PROGMEM = {0x6D,0x18,0x49,0x91,0xBC,0x17,0xEF,0x6E,0x15,0xA3,0x15,0xA2,0xE5,0x93,0x9D,0xB5,0x7C,0x6C,0x07,0xB6,0x7C,0x1C,0xF2,0x11,0x19,0xAC,0xB2,0x0E,0x02,0x45,0x28,0x77,0x11,0xD9,0x00,0x04,0xF0,0xA3,0x88,0x01,0xBE,0x65,0xB4,0x36,0xC8,0x8D,0x08,0xF4,0x33,0xBB,0x39,0xB4,0xB5,0xE2,0xAE,0x0E,0xF2,0xDB,0xD7,0xBA,0xA7,0x23,0xD3,0xEA,0x0E,0xF0,0x9B,0x8E,0xC8,0xAE,0x92,0x24,0x77,0x38,0x33,0xF8,0x68,0xE6,0xD6,0xF1,0x4C,0xD7,0x25,0x21,0xE2,0x92,0x37,0x5D,0x17,0xB1,0x9B,0x87,0x9B,0x7C,0xB7,0xFB,0xFB,0x08,0x13,0x66,0xA6,0x80,0xEF,0x91,0x14,0xB0,0x21,0xE7,0xFF,0x03};
uint8_t const spTIMES[] PROGMEM = {0x06,0x18,0xD2,0xCD,0x01,0x9B,0xAB,0x69,0x60,0x15,0xD5,0x74,0x44,0xF2,0x9A,0x55,0x15,0x3B,0xEE,0x29,0x4B,0x76,0xB5,0xEA,0xE2,0xC7,0xBD,0x64,0xC9,0xAA,0x4B,0x18,0xF3,0x94,0x25,0xAB,0x2B,0x61,0xCC,0x53,0x96,0xAE,0xA9,0xC6,0x56,0x4F,0x59,0x32,0xA6,0x9A,0x5D,0x23,0x7C,0xCE,0x58,0x6B,0x54,0x0B,0xCF,0xC9,0x6D,0xAF,0xDE,0x24,0x2A,0x26,0x95,0xBD,0x49,0x61,0x0F,0x8D,0x97,0x8F,0x74,0x56,0x29,0xEC,0x6E,0x55,0xF3,0x69,0x8E,0x26,0x44,0xCC,0xAD,0xAA,0x39,0x9A,0x14,0x71,0x93,0x6A,0xE6,0xAC,0x86,0xCD,0xD5,0xE2,0xA6,0xEB,0x92,0x10,0x71,0xC9,0x9B,0xAE,0x8B,0xD8,0xCD,0xC3,0x4D,0xBE,0xD3,0xFD,0x7D,0x84,0x09,0x33,0x53,0xC0,0xF7,0x48,0x0A,0xD8,0x90,0xF3,0xFF,0x01};
uint8_t const spDIVIDED[] PROGMEM = {0x0A,0x28,0x30,0x6C,0x2C,0x43,0xAA,0x8B,0x64,0xA4,0xD3,0x0F,0xE5,0x22,0x26,0x5B,0x4E,0xD7,0x8C,0xAB,0x98,0x6D,0x39,0x7D,0xD3,0x66,0x1A,0xDC,0xA5,0x4D,0x85,0x99,0xA9,0x71,0x96,0x36,0x57,0xA5,0x6A,0x62,0x71,0x4E,0x5B,0x64,0xA5,0x07,0xA5,0x59,0x55,0x72,0x95,0x61,0xBC,0xF9,0xA4,0xD9,0x95,0xA7,0xC9,0xE6,0x11,0x95,0x90,0x56,0x62,0xAB,0x5B,0x5A,0xA2,0x6B,0x4A,0xCE,0x2E,0x59,0x73,0x2E,0xC1,0xB9,0xBA,0xB5,0x09,0x9B,0x45,0x54,0xEB,0x36,0x54,0xAD,0x14,0xBC,0x73,0xDA,0xD4,0x8C,0x51,0x48,0x2F,0x6E,0x4B,0x53,0x4E,0xAE,0xB9,0x9A,0x1D,0x8E,0x64,0xA6,0x45,0xA3,0xFF,0x07};

uint8_t const spPOINT[] PROGMEM = {0x0A,0xC8,0x2E,0x8C,0x01,0xD1,0x85,0x95,0x34,0x8B,0xAA,0x74,0x56,0xBD,0xCA,0xE8,0xA6,0xCA,0x29,0xCB,0x2A,0x92,0xEE,0xCA,0xA4,0x36,0x2B,0x2F,0xAA,0x3C,0x43,0x9A,0xAC,0xBC,0x78,0xF3,0xD0,0x68,0xD4,0xB2,0x6A,0x45,0x4A,0x2A,0xA6,0xC9,0x93,0xC4,0x29,0x09,0xC7,0x24,0x8D,0xB1,0xD3,0xCD,0x08,0x21,0x70,0x8A,0x52,0x15,0xAA,0x01,0xE0,0x80,0x63,0xC2,0x14,0x70,0x94,0x19,0x01,0xA6,0x32,0xFB,0x7F};

uint8_t const spGOODBYE[]           PROGMEM ={0xA9,0x49,0xE1,0x54,0x91,0x2D,0xAF,0x22,0x07,0x55,0x29,0x69,0x7B,0xF2,0x18,0x38,0x32,0x3C,0xCB,0x4D,0x52,0xC8,0x4A,0x5A,0x65,0x99,0x52,0x21,0x6A,0x61,0x69,0x2E,0x45,0x46,0x2C,0x43,0xA9,0x3C,0x3D,0x1C,0x87,0x2A,0x3A,0xFB,0x50,0x6E,0x73,0xEB,0xDB,0xEC,0x6E,0x77,0xA3,0xF7,0xD1,0x4E,0x35,0xA2,0xCC,0x7E,0x74,0x3E,0xC6,0xA8,0x79,0x85,0xD1,0x86,0x64,0x65,0x16,0x95,0x5B,0x13,0x52,0xB0,0x65,0x94,0x2E,0xB5,0x4D,0x6A,0x55,0x65,0xF8,0x01,0x00,0x00};
uint8_t const spREADY_TO_START[]    PROGMEM ={0xAE,0x91,0x85,0xD3,0x32,0xB8,0xB5,0xC6,0x35,0x4E,0x09,0xCA,0x7E,0x6A,0xDF,0xB9,0xCC,0xB4,0xFB,0xA9,0xE2,0xD4,0x30,0xD5,0xEE,0xB7,0x5A,0x55,0xEA,0x5A,0x10,0x99,0xA5,0x55,0x31,0x1B,0x1B,0x4E,0xA6,0x55,0xE5,0xAE,0xCA,0xD2,0x9D,0x5A,0x55,0xA6,0x08,0x49,0x2D,0x8D,0xB5,0x6C,0x11,0xB0,0x54,0x9A,0x06,0x0C,0xB0,0x8C,0x45,0x29,0xBC,0x4A,0x53,0xCB,0x4E,0xA5,0xA9,0xC2,0x4C,0xD5,0xF2,0x1A,0xA0,0x03,0x35,0x05,0xFC,0xA9,0x6C,0x01,0x05,0xFC,0x1D,0x0C,0x10,0x80,0xA6,0xD2,0x4F,0x1E,0x43,0x44,0x86,0xB4,0xBD,0xC5,0xAD,0x4E,0xE5,0x8B,0x77,0x18,0x37,0x59,0x85,0x9F,0xE6,0xA5,0xBA,0x78,0x67,0x33,0x6D,0x91,0x9F,0x22,0x1D,0xE6,0xD0,0x45,0xDA,0x49,0x46,0xB8,0x2C,0x00,0x03,0x64,0x19,0xCA,0x80,0x26,0x9B,0x11,0xE0,0x12,0x33,0x00,0xC0,0x03,0x00,0x00};

#define MaxDigits 7                                    // MaxDigits = Displays - 1
#define DoSerial false
#define useWDT   true

// -----------------------------------------------------------------------------------------------------------------

volatile float firstNumber = 0.0;
volatile float secondNumber = 0.0;

//volatile float tempFLOAT = 0;                       // *** Changed to save memory
//  Reuse  firstNumber inplace

volatile long tempLONG = 0;

long working = 0;

//unsigned long currentMillis;                           // *** Changed to save memory
// Now just using millis()
unsigned long SleepTimer;
unsigned long ClearTimer;
bool talking = true;
bool female = true;
// unsigned long SleepLimit = 15000;                    // *** Changed to save memory
#define SleepLimit 15000

volatile int mode = 0;                                        // Must be int, <0 is used

volatile int commandType = 0;                        // Where 1="+", 2="-", 3="/", 4="*", 5="="
volatile int runoncommand = 0;


volatile int tempINT = 0;

byte i = 0;

byte MSDIndex = 0;
byte LSDIndex = 0;
byte LeftIndex = 0;

volatile byte DPDEX = 0;                              // Start at first decimal point

byte currentKey = 0;
// byte result = 0;                                   // *** Changed to save memory                     

byte disp[9] = {
  0, 10, 10, 10, 10, 10,10,10,10};                             // 0 to 9 show, 10 = display off, 11 = "-" symbol, 12 = "E" for error
byte MSD[9]= {
  0, 0, 0, 0, 0, 0,0,0,0};
byte LSD[9]= {
  0, 0, 0, 0, 0, 0,0,0,0};

byte column = 0;
byte maxInput = 8;
byte rawKeys = 0;
byte KeyValue = 0;
volatile byte ErrorCode = 0;                         // Where 1 = number too big (ie more than 6 digits)
//       2 = WDT overflow
//       3 = Brown-out
//       4 = Switch case jump too high
//       5 = External reset
//       6 = Power On reset

volatile boolean ErrorFlag = false;
boolean negativeFlag = false;
boolean decplace = false;                             // T or F just used to turn DP LED on or off
boolean commandFlag = false;

boolean decimalFlag = false;                          // Decimal place key press
boolean KeyPress = false;
boolean clearkeyval = false;
boolean KeyOK = true;
boolean FirstNumberStart = false;
boolean SecondNumberStart = false;
boolean ClearFlag = false;

boolean secondNegativeFlag = false;
boolean DPsecondNumbFlag = false;

boolean firstNegativeFlag = false;
boolean DPfirstNumbFlag = false;

boolean leadingDig = false;

boolean runoncalc = false;

volatile boolean KeyDown[18];
#define speaker   3

byte digit[8] = {14,15,16,17,18,19,0,1};
// -----------------------------------------------------------------------------------------------------------------

void setup() 
{ 
  pinMode(2, INPUT_PULLUP);                                 // CLEAR / ON BUTTON (USED TO WAKE CALCULATOR)
  pinMode(speaker, OUTPUT);                          // 2nd Function button selector (This pin is currently not connected)

  DDRB = DDRB | 1 << PORTB0;                         // SEGMENT A
  DDRB = DDRB | 1 << PORTB1;                         // SEGMENT B
  DDRB = DDRB | 1 << PORTB2;                         // SEGMENT C 
  DDRB = DDRB | 1 << PORTB3;                         // SEGMENT D
  DDRB = DDRB | 1 << PORTB4;                         // SEGMENT E
  DDRB = DDRB | 1 << PORTB5;                         // SEGMENT F 
  DDRB = DDRB | 1 << PORTB6;                         // SEGMENT G 
  DDRB = DDRB | 1 << PORTB7;                         // SEGMENT DECIMAL PLACE

  pinMode(4, INPUT);                                 // KEYPAD ROW 0 (TOP ROW)
  pinMode(5, INPUT);                                 // KEYPAD ROW 1
  pinMode(6, INPUT);                                 // KEYPAD ROW 2 
  pinMode(7, INPUT);                                 // KEYPAD ROW 3 (BOTTOM ROW)

  digitalWrite(4,LOW);                               // This is just an attemp to reduce power use
  digitalWrite(5,LOW);                               // These pins are pulled down to GND with 10K resistors
  digitalWrite(6,LOW);
  digitalWrite(7,LOW);

  pinMode(A0, OUTPUT);                               // KEYPAD COLUMN 0 (RIGHT MOST) AND DIGIT 0 (RIGHT MOST)
  pinMode(A1, OUTPUT);                               // KEYPAD COLUMN 1 AND DIGIT 1 
  pinMode(A2, OUTPUT);                               // KEYPAD COLUMN 2 AND DIGIT 2
  pinMode(A3, OUTPUT);                               // KEYPAD COLUMN 3 AND DIGIT 3 
  pinMode(A4, OUTPUT);                               // DIGIT 4
  pinMode(A5, OUTPUT);                               // DIGIT 5
  pinMode(0, OUTPUT);                               // DIGIT 6
  pinMode(1, OUTPUT);                               // DIGIT 7
  
#if DoSerial
  Serial.begin(9600);  
#endif

  ClearAllKeys();

  FirstNumberStart = true;

  SleepTimer = millis();

  // Error reporting ------------------------------------------------------------------------------- S
  // Where 1 = number too big (ie more than 6 digits)
  //       2 = WDT overflow
  //       3 = Brown-out
  //       4 = Switch case jump too high
  //       5 = External reset
  //       6 = Power On reset
  //       7 = WDT System reset
  //       9 = Multiple Errors

    tempINT = MCUSR & B00001111;                        // RES,RES,RES,RES,WDRF,BORF,EXTRF,PORF

  if(tempINT > 0)
  { 
    ErrorFlag = true; 
    switch(tempINT)
    {
    case 1:        
      ErrorCode = 6; 
      break;  

    case 2:
      ErrorCode = 5; 
      break;

    case 4:
      ErrorCode = 3; 
      break;

    case 8:
      ErrorCode = 7; 
      break;

    default:
      //      ErrorCode = 9;
      if(bitRead(tempINT,0))
      {
        ErrorCode = 6;                              // Power on
      }
      else
      {
        ErrorCode = 9;                              // Other multiple errors
      }
      break;
    }
  }

  if(bitRead(WDTCSR, WDIF))
  {
    ErrorFlag = true;
    ErrorCode = 7; 
    bitSet(WDTCSR, WDIF);                        // Write a 1 to clear ???
  }

  MCUSR = 0;
  // Error reporting ------------------------------------------------------------------------------- E

#if useWDT
  WDT_Init();
#endif 

}

// -----------------------------------------------------------------------------------------------------------------


void loop() 
{

#if useWDT  
  wdt_reset(); 
#endif

  if((millis() - SleepTimer) > SleepLimit)
  {
    GoToSleep();
  }

  UpdateDisplayCheck();                                    // Check for button press and Update Display LEDs

  if(ErrorFlag)
  {
    mode = 4;
  }
    if(KeyPress)  
    {
if (talking==true)
{
  // commandType => 1="+", 2="-", 3="/", 4="*", 5="=", 6="Clear"
        if(commandFlag == false)
    {
    sayNumber(KeyValue);
    }
            if(commandFlag == true)
            {
    switch (KeyValue) {
  // commandType => 1="+", 2="-", 3="/", 4="*", 5="=", 6="Clear"
    case 1: 
      talkieSay(spPLUS); 
      break;
    case 2: 
      talkieSay(spMINUS); 
      break;
    case 3: 
      talkieSay(spDIVIDED);  
      break;
    case 4: 
      talkieSay(spTIMES);  
      break;
    case 5: 
      talkieSay(spEQUALS);  
      break;      
    
    default: 
      break; // Say nothing more for numbers like 20 or 50
    }
    }
    ///--------------------------------------
    }
}
  switch (mode)                                            // Calculator working statemachine
  {
  case 0:                                                  // Building first number
    if(KeyPress)                                           // Any key is pressed
    {
      SleepTimer = millis();
      if(KeyOK == true)                                    // NEW key is pressed
      {
 ///   playTone( 10, 1000);
        if(commandFlag == false)                           // Key is not a command
        {
          if(FirstNumberStart && (KeyValue > 0))           // New number being entered, remove leading zero
          {
            BlankDisplay();
            FirstNumberStart = false;
            DPDEX = 0;
            if(decimalFlag)                                // Start with a decimal place
            {
              disp[0] = 0;
            }
          } 

          if(LeftIndex < maxInput && !FirstNumberStart && !decimalFlag)                               // Display is not full (max six digits)
          {
            for(i = 8; i >0; i --)
            {
              disp[i] = disp[i-1];
            }
            disp[0] = KeyValue;

            if(DPfirstNumbFlag)
            {
              firstNumber =(float(KeyValue)/pow(10.0,(float(DPDEX)+1.0))) + firstNumber;       // Append key value to number being "built"
            }
            else
            {
              //              firstNumber =(float)KeyValue + (firstNumber*10);                               // Jan 20th
              tempLONG = long(KeyValue) + (tempLONG*10L);
            }

            LeftIndex = LeftIndex +1;

            if(DPfirstNumbFlag)
            {
              DPDEX = DPDEX + 1;
            }

#if DoSerial
            TestPrints();                                                      // Just for testing
#endif  

          }

          if(decimalFlag)
          {
            DPfirstNumbFlag = true;
            // floatFlag = true;
            decimalFlag = false;
            maxInput=6;
          }
        }
        else
        {
          if(FirstNumberStart)
          {
            if(KeyValue ==2)                                                     // Starting to build a negative number
            {
              firstNegativeFlag = true;
              negativeFlag = true;
              maxInput=6;
            }
            // else do nothing - user pressed "silly" command
          }
          else
          {

            //          negativeFlag = false;          
            BlinkFeedback();                                                     // Command key pressed, blink display

            if(KeyValue !=5)                                                     // "=" NOT pressed do nothing
            {
              mode = 1;                                                            // Set command
              firstNumber = float(tempLONG) + firstNumber;                                // Jan 20th - recombine decimal with long
              tempLONG = 0;
              secondNumber = 0.0;                                                  // J21
              SecondNumberStart = true;
              secondNegativeFlag = false;
              LeftIndex = 0;
              commandType = int(KeyValue);
              // Serial.print("Command Pressed !!! :");
              // Serial.println(commandType);
            }
          } 
          commandFlag = false;                                                  

#if DoSerial
          TestPrints();                                                      // Just for testing
#endif 
        }
      }
    }
    break;

  case 1:                                                                         // Building Second number

    if(KeyPress)
    {
      SleepTimer = millis();
      if(KeyOK == true)
      {
        if(commandFlag == false)
        {
          if(SecondNumberStart && (KeyValue > 0))                                 // First digit being entered (but not a zero)
          {
            BlankDisplay();
            SecondNumberStart = false;
            if(!secondNegativeFlag)
            {
              negativeFlag = false;

            }
            DPDEX = 0; 
            if(decimalFlag)                                                        // Start with a decimal place
            {
              disp[0] = 0;
            }
          }  


          if(LeftIndex < maxInput && !SecondNumberStart && !decimalFlag)
          {
            for(i = 8; i >0; i --)                                                 // Shift display to left as new numbers added
            {
              disp[i] = disp[i-1];
            }
            disp[0] = KeyValue;


            if(DPsecondNumbFlag)
            {
              secondNumber =(float(KeyValue)/pow(10.0,(float(DPDEX)+1.0))) + secondNumber;                    // Append key value to number being "built"
            }
            else
            {          
              //              secondNumber =(float)KeyValue + (secondNumber*10);
              tempLONG = long(KeyValue) + (tempLONG*10L);
            }


#if DoSerial
            TestPrints();                                                      // Just for testing
#endif 

            LeftIndex = LeftIndex +1;


            if(DPsecondNumbFlag)
            {
              DPDEX = DPDEX + 1;
            }
          }

          if(decimalFlag)
          {
            DPsecondNumbFlag = true;
            decimalFlag = false;
            //floatFlag = true;
              maxInput=6;
          }

        }
        else
        {
          if(SecondNumberStart)
          {
            if(KeyValue ==2)                                                      // Starting to build a negative number
            {
              BlankDisplay();                                                      // Starting with a negative number -> remove firstnumber from display
              disp[0] = 0;
              secondNegativeFlag = true;
              negativeFlag = true;
              commandFlag = false; 
              maxInput=6;
            }                                                                      // User typed a command before any value = do nothing
          }
          else
          {

            BlinkFeedback();                                                      // *** Should test which command is pressed and not just perform "="

            if(KeyValue ==5)
            {
              runoncalc = false;                                                  // = was pressed
            }
            else
            {              
              runoncalc = true;                                                   // Another command was entered, get ready for more work
              runoncommand = int(KeyValue);
            }

            commandFlag = false; 
            mode = 2;
            secondNumber = float(tempLONG) + secondNumber;
            tempLONG = 0;
            LeftIndex = 0;
            BlankDisplay();    
          }  
          commandFlag = false;                                                  // New Jan 16th 2013
        }
      }
    }  

    break;

  case 2:                                                                        // Do Computation   // Where 1="+", 2="-", 3="/", 4="*", 5="=

    if(secondNegativeFlag)
    {
      secondNumber = 0 - secondNumber;
    }

    if(firstNegativeFlag)
    {
      firstNumber = 0 - firstNumber;
    }

#if DoSerial
    Serial.println("Before Calc:");
    Serial.println(firstNumber, 6);
    Serial.println(secondNumber, 6);
#endif

    switch(commandType)
    {
    case 1:                                                        // Addition
      firstNumber = firstNumber+secondNumber;
      break;

    case 2:                                                        // Subraction
      firstNumber = firstNumber-secondNumber;
      break; 

    case 3:                                                        // Division
      firstNumber = firstNumber/secondNumber;
      //floatFlag = true;
      break;      

    case 4:                                                         // Multiplication
      firstNumber = firstNumber * secondNumber;
      break;

    default: 
      firstNumber = 1.0;                                            // Debugging only
      ResetClearAll(); 
      ErrorFlag = true;
      ErrorCode = 4; 
    }

    if((long(firstNumber) > 9999999999L) || (long(firstNumber) < -999999999L)) //99999999L) || (long(firstNumber) < -9999999L)
    {
      //      exponentresult = true;
      ErrorFlag = true;
      ErrorCode = 1;
    }


#if DoSerial
    TestPrints();                                                      // Just for testing
#endif 

    FloatToDigits();                                                 // Always use floats
//    playTone( 30, 5000);


        if ((negativeFlag)&&(talking==true))
          {
            talkieSay(spMINUS); 
          }

  for(byte q = 8; q>0;q--)                                         // Clear the array before reloading it
  {
    sayNumber(disp[q-1]);
      if((DPDEX == (q-1))&&(talking==true)&&(q!=1))
  {
            talkieSay(spPOINT); 
  }
  }
      //  mode = 3;                                                      // was mode = 0
    //    secondNumber = 0.0;
    //    SecondNumberStart = true;

    if(negativeFlag)
    {
      firstNegativeFlag = true;
    }

    if(runoncalc)                                                   // This happens when user presses +-/* at rather than = after 2nd number
    {
      runoncalc = false;
      commandType = runoncommand;
      secondNumber = 0.0;
      SecondNumberStart = true;                                     
      secondNegativeFlag = false;
      DPsecondNumbFlag = false;
      LeftIndex = 0;
      tempLONG = 0;                                                 // J21
      mode = 1;                                                     // Go back to building second number using command from 1st 2nd number built exi

    }
    else
    {
      mode = 3;                                                     // = was pressed after 2nd number built
      tempLONG = 0;
    }

    break;  

  case 3:                                                            // Using result in next calculation / or just displaying last result and waiting
      if(KeyPress)
    {
      SleepTimer = millis();
      if(KeyOK == true)
      {
        if(commandFlag == true)
        {
          BlinkFeedback();                                           // Command key pressed, blink display
          if(int(KeyValue) == 5)  
          {
            mode = 2;                                                // "=" pressed, redue calc with same last numbers
            tempLONG = 0;
            LeftIndex = 0;
            BlankDisplay();
          }
          else
          {
            commandType = int(KeyValue);
            mode = 1;                                                // Build second number - again, use firstnumber on display
            tempLONG = 0;                                            // J21
            secondNumber = 0.0;
            SecondNumberStart = true;
            secondNegativeFlag = false;
            DPsecondNumbFlag = false;
            // DPDEX = 0;
            LeftIndex = 0;
          }
          commandFlag = false;  
        }
        else                                                          // Re-build a first number
        {
          BlankDisplay();
          FirstNumberStart = false;
          tempLONG = 0;
          mode = 0;
          DPDEX = 0;
          if(decimalFlag)                                             // Start with a decimal place
          {
            disp[0] = 0;
            DPDEX = 1;
            DPfirstNumbFlag = true;
            decimalFlag = false;
          }
          else
          {
            tempLONG = long(KeyValue);                                                        // J21
            //            firstNumber =(float)KeyValue;
            disp[0] = KeyValue;
            LeftIndex = 1;
          }
        }

      }
    } 

    break;

  case 4:

    switch(ErrorCode)
    {
    case 1:                                                      // Number too big or small
/*    
      disp[0] = 21;  
      disp[1] = 22;    
      disp[2] = 17;
      disp[3] = 20;
      disp[4] = 20;
      disp[5] = 19;
*/      
      disp[0] = 11;                                              // rAnGE
      disp[1] = 21;    
      disp[2] = 23;
      disp[3] = 16;
      disp[4] = 12;
      disp[5] = 10; 
      disp[6] = 0;
      disp[7] = 0; 
      break;

    case 6:                                                      // Power ON Splash Spells out "CAL_1.1"
      disp[0] = 10; 
      disp[1] = 10;    
      disp[2] = 18;
      disp[3] = 15;
      disp[4] = 16;
      disp[5] = 18;    
      disp[6] = 10;
      disp[7] = 10; 
      break;

    case 3:                                                      // Brown-out = battery low
      disp[0] = 20;
      disp[1] = 15;    
      disp[2] = 10;
      disp[3] = 19;
      disp[4] = 16;
      disp[5] = 17;
      disp[6] = 10;
      disp[7] = 10;     
      break;  

    case 5:                                                      // Reset Pin
      disp[0] = 19;
      disp[1] = 11;    
      disp[2] = 5;
      disp[3] = 11;
      disp[4] = 12;
      disp[5] = 10; 
      disp[6] = 10;
      disp[7] = 10;    
      break; 

    default:                                                      // Show other error number
      disp[0] = ErrorCode;
      disp[1] = 10;    
      disp[2] = 10;
      disp[3] = 12;
      disp[4] = 12;
      disp[5] = 11;
      disp[6] = 10;
      disp[7] = 10; 
      break; 
    }

    break;

  default:                                                      // Should never get here
    ResetClearAll(); 
    ErrorFlag = true;
    ErrorCode = 4; 
    break;     

  }
}

// -----------------------------------------------------------------------------------------------------------------

void UpdateDisplayCheck()
{
  PORTC = B00000000;
  PORTB = B00000000;

   digitalWrite(digit[column], HIGH);   //column+14                           // Turn off current Digit
  

  if(column <4)
  {
    rawKeys = (PIND & B11110000) >>4;                        // Test here for button Press - Read keys port value
    //  rawKeyArray[column] = (PIND & B11110000) >>4; 
    //    oldrawkeys[column] = (PIND & B11110000) >>4;
  }
  else
  {
    rawKeys = 0;
  }
  if(rawKeys>0)                                                // Key being pressed
    //  if(rawKeyArray[column]>0 && (rawKeyArray[column] == oldrawkeys[column]))
  {
    decodeKeyPress();
    KeyPress = true;
  }
  else
  {
    KeyPress = false;

    if(column < 4)
    {
      for(i = (column*4); i<((column*4)+4); i++)                 // Clears the KeyDown variable to signal a key release. Only for current column.
      {
        KeyDown[i] = false;
      }
    }
  }
   column = column +1; 
  if(column >MaxDigits)                                      // > 4 or MaxDigits
  {
    column = 0;
  }
  PORTC = B11111111;
  digitalWrite(digit[column], LOW);                             // Power-up Column (both for buttons and LEDs)
  delayMicroseconds(300);                                   // Better debouncing ? (Dimmer display ?) 1000 is good



    
  if(DPDEX == column)
  {
    decplace = true;
  }
  else
  {
    decplace = false;
  }


  if(disp[column] >9 && !ErrorFlag)                       // Display nothing
  {
    PORTB = B00000000;
    if((column == MaxDigits) && negativeFlag)
    {
      PORTB = B01000000;                                       // Negative symbol
    }
  }
  else
  {    
    //    PORTB = decodeSegments(disp[column], decplace);            // Set 7 Segements on / off 
    //    PORTB = SYMBOLS[disp[column]] ^ decplace <<7; 
    SegmentDecode();      
  }



  //  if(exponentresult)
  //  {

  //  PORTB = B10000110; 
  //   SegmentDecode();
  //   mode = 4;
  //  }

  delayMicroseconds(400);                                      // For a brighter display ? 400 is good

  //  rawKeyArray[column] = (PIND & B11110000) >>4;



  // delayMicroseconds(250);                                   // For a brighter display ?


  // -----------------------------------------------------------------------------------------------------------------

  //  Test for Clear Key press - this was it's own void before

    clearkeyval = digitalRead(2);

  if(!clearkeyval)
  {
    ClearFlag = true;
    SecondNumberStart = true;
    secondNumber = 0.0;    
    secondNegativeFlag = false;
    negativeFlag = false;
    ClearTimer = millis();
    PORTB = B00000000;

    do                                                                  // Wait for CLEAR key to be released
    {
      delay(100);          
      clearkeyval = digitalRead(2);
    } 
    while (!clearkeyval);
    if ((millis()-ClearTimer) > 1000)
    {
    playTone( 10, 5000);
    if (talking==true) talking=false; else
    {
    talking=true;
    playTone( 80, 5000);
    if (female==true) female=false; else 
    {
    playTone( 80, 8000);
    female=true;
    }
    }
    }
    SleepTimer = millis();

    mode = mode - 1;
    if(mode<0 || mode == 2 || ErrorFlag)                          // CLEAR ALL (Interested in mode = 3 but we do a mode --
    {
      ResetClearAll();

      disp[0] = 0;
      for(i = 1; i<8; i++)
      {
        disp[i] = 10;
      }
    }

    secondNumber = 0.0;
    LeftIndex = 0;
    column = 0;
  }
  else
  {
    ClearFlag = false;
  }
}

// -----------------------------------------------------------------------------------------------------------------
void SegmentDecode()
{
  switch (disp[column]) 
  {
  case 0:    
    PORTB =  B00111111;
    break;

  case 1:    
    PORTB =  B00000110;
    break;

  case 2:    
    PORTB =  B01011011;
    break;

  case 3:    
    PORTB =  B01001111;
    break;

  case 4:    
    PORTB =  B01100110;
    break;

  case 5:    
    PORTB =  B01101101;
    break;

  case 6:    
    PORTB =  B01111101;
    break;

  case 7:    
    PORTB =  B00000111;
    break;

  case 8:    
    PORTB =  B01111111;
    break;

  case 9:    
    PORTB =  B01100111;  
    break; 

  case 10:                                                      // Blank space
    PORTB =  B00000000;  
    break;    

  case 11:                                                      // "E"
    PORTB =  B01111001;  
    break; 

  case 12:                                                      // "r"
//    PORTB =  B10101111;  

    PORTB =  B00110001; 
    break; 

  case 13:                                                      // "S"
    PORTB =  B01101101;  
    break; 

  case 14:                                                      // "P"
    PORTB =  B01110011;  
    break;

  case 15:                                                      // "L"
    PORTB =  B00111000;  
    break;

  case 16:                                                      // "A"
    PORTB =  B01110111;  
    break;

  case 17:                                                      // "b"
    PORTB =  B01111100;  
    break;

  case 18:
 //   PORTB = B11000110;                                            // C  
    PORTB = B00111001;                                            // C  
    break; 

  case 19:
    PORTB = B01111000;                                            // "t"  
    break;   

  case 20:
    PORTB = B01011100;                                            // "o"  
    break;

  case 21:
    //    PORTB = B10010000;                                      // "g"  
    PORTB = B00111101;                                            // 'G"
    break;

  case 22:
        PORTB = B00010000;                                     // "i"  
    break;  

  case 23:
//       PORTB = B10101011;                                     // "n"  
        PORTB = B00110111;                                      // N
    break;  

  default:                                                      // Should never get here  
    PORTB =  B00000000;
    ErrorFlag = true;
    ErrorCode = 4; 
    break;
  }


  if(decplace)
  {
    bitSet(PORTB, 7);
  }
  else
  {
    bitClear(PORTB, 7);
  }

}

// -----------------------------------------------------------------------------------------------------------------

void decodeKeyPress()
{
  tempINT = 0;                                                        // New Dec 20th

  // KeyPad Layout
  //
  // Columns      Rows
  //   3 2 1 0
  //  
  //   7 8 9 /    0
  //   4 5 6 *    1
  //   1 2 3 -    2
  //   . 0 = +    3

  // Value mapping for decoding
  //     12  8  4  0
  //     13  9  5  1
  //     14  10 6  2
  //     15  11 7  3

  // commandType => 1="+", 2="-", 3="/", 4="*", 5="=", 6="Clear"

  for (i=0; i < 4; i++)                                                // This decodes the Rows
  {
    if(bitRead(rawKeys, i))
      //if(bitRead(rawKeyArray[column], i))   
    {
      tempINT = i;
      break;
    }
  } 

  currentKey = ((column)*4)+tempINT;

  if(KeyDown[currentKey] == false)
  {

    KeyOK = true;
    KeyDown[currentKey] = true;

    switch (currentKey) 
    {
    case 0:    
      KeyValue =  3;
      commandFlag = true;
      break;

    case 1:    
      KeyValue =  4;
      commandFlag = true;
      break;

    case 2:    
      KeyValue =  2;
      commandFlag = true;
      break;

    case 3:    
      KeyValue =  1;
      commandFlag = true;
      break;

    case 4:    
      KeyValue =  9;
      break;

    case 5:    
      KeyValue =  6;
      break;

    case 6:    
      KeyValue =  3;
      break;

    case 7:    
      KeyValue =  5;
      commandFlag = true;
      break;

    case 8:    
      KeyValue =  8;
      break;

    case 9:    
      KeyValue =  5;
      break; 

    case 10:    
      KeyValue =  2;
      break;

    case 15:    
      KeyValue =  11;
      decimalFlag = true;     
      break;  

    case 12:    
      KeyValue =  7;
      break;

    case 13:    
      KeyValue =  4;
      break; 

    case 14:    
      KeyValue =  1;
      break;

    case 11:    
      KeyValue =  0;
      break;  

      //    case 16:                                              // Clear
      //      KeyValue =  6;
      //      commandFlag = true;
      //      break; 
    default:                                                      // Should never get here
      KeyOK = false;
      KeyValue =  0;
      ErrorFlag = true;
      ErrorCode = 4; 
      //  KeyDown[currentKey] = false;
    }
  }
  else
  {
    KeyOK = false;                                                // Not a new key being pressed
  }
}


// -----------------------------------------------------------------------------------------------------------------

void BlinkFeedback()
{  
  PORTB = B00000000;
  delay(200);  
}

// -----------------------------------------------------------------------------------------------------------------

void BlankDisplay()
{
  PORTB = B00000000;
  for(i = 0; i <9; i ++)
  {
    disp[i] = 10;
  }
}

// -----------------------------------------------------------------------------------------------------------------

void ClearAllKeys()
{
  for(i =0; i<17; i++)
  {
    KeyDown[i] = false;
  }
}

// -----------------------------------------------------------------------------------------------------------------

void ResetClearAll()
{
  ClearAllKeys();
//      playTone( 300, 5000);
  PORTB = B00000000;
  for(i = 0; i <9; i ++)
  {
    disp[i] = 10;
  }

  DPfirstNumbFlag = false;
  firstNumber = 0.0;
  DPsecondNumbFlag = false;
  secondNumber = 0.0;
  mode = 0;
  tempINT = 0;
  tempLONG =0;
  //  tempFLOAT = 0.0;
  column = 0;
  rawKeys = 0;
  KeyValue = 0;
  LeftIndex = 0;
  DPDEX = 0;
  // floatFlag = false;
  decplace = false;
  //  exponentresult = false;
  ErrorFlag = false;
  ErrorCode = 0;
  commandFlag = false;
  commandType = 0;
  decimalFlag = false;
  KeyPress = false;
  KeyOK = true;
  FirstNumberStart = true;
  SecondNumberStart = true;
  secondNegativeFlag = false;
  firstNegativeFlag = false;
  ClearFlag = false;
  currentKey = 0;  
  negativeFlag = false;
  disp[0] = 0;
  maxInput=8;
  MCUSR = 0;                                    // ATmega error flag bits
}

// -----------------------------------------------------------------------------------------------------------------


void TestPrints()
{
  i = 0;                                        // Basic do nothing

#if DoSerial
  Serial.println("");
  Serial.print("First number (4 digits) : ");
  Serial.println(firstNumber, 4);

  Serial.print("First number (6 digits) : ");
  Serial.println(firstNumber, 6);

  Serial.print("TEMP LONG: ");
  Serial.println(tempLONG);

  Serial.print("Error Code :");
  Serial.println(ErrorCode);


  Serial.print("First number LONG: ");
  Serial.println(long(firstNumber));

  Serial.print("Second number: ");
  Serial.println(secondNumber, 4);

  Serial.print("Second number (6 digits) : ");
  Serial.println(secondNumber, 6);

  Serial.print("TEMP LONG: ");
  Serial.println(tempLONG);

  Serial.print("Command: ");
  Serial.print(" - ");
  Serial.println(commandType);

  Serial.print("Mode: ");
  Serial.println(mode);

  Serial.print("Clear: ");
  Serial.println(ClearFlag);

  //  Serial.print("Start: First = ");
  //  Serial.print(FirstNumberStart);      
  //  Serial.print(" : Second = ");
  //  Serial.println(SecondNumberStart); 

  Serial.print("KeyPress: ");
  Serial.println(KeyPress);

  Serial.print("Key OK: ");
  Serial.println(KeyOK);

  Serial.print("Left Index: ");
  Serial.println(LeftIndex);

  Serial.print("DP Index: ");
  Serial.println(DPDEX);

  Serial.print("First Decimal: ");
  Serial.println(DPfirstNumbFlag);

  Serial.print("WDTCSR: ");
  Serial.println(WDTCSR, BIN);

  Serial.print("MCUSR: ");
  Serial.println(MCUSR, BIN);

  Serial.print("freeMemory()=");
  Serial.println(freeMemory());

#endif  
}

// -----------------------------------------------------------------------------------------------------------------

void GoToSleep()
{
//        talkieSay(spGOODBYE);
#if useWDT  
  wdt_reset();  
  wdt_disable();
#endif  

  PORTC = PORTC | B00111111;                                    // Also set all columns to high (reduce reverse current waste)

  PORTB = B00000000;                                            // Clear all LEDs (Cathod side)

//  power_timer1_disable();                                       // 
//  power_timer0_disable();                                       // 
//  power_timer2_disable();                                       // 

  power_twi_disable();
  power_usart0_disable();
  power_spi_disable();

  ADCSRA &= ~(1<<ADEN);                                         // Turn off analog
  ACSR = (1<<ACD);                                              // Turn off comparator

  attachInterrupt(0, ClearON, RISING);
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);                          //   was   (SLEEP_MODE_EXT_STANDBY);
  cli();

  sleep_enable();

  sei();
  sleep_cpu();
  //    _NOP();
  //    _NOP();
  sleep_disable();

  sei();

#if DoSerial
  power_usart0_enable();
#endif 
  power_timer0_enable();

  detachInterrupt(0);

  PORTC = PORTC & B11000000;                                    // Turn-off oll columns (as to not light any LEDs unintentionaly)
  SleepTimer = millis();

#if useWDT  
  //  wdt_reset();
  //  wdt_enable(WDTO_2S);
  WDT_Init();
#endif  
//playTone( 30, 5000);
//        talkieSay(spREADY_TO_START);
  ResetClearAll();
}

// -----------------------------------------------------------------------------------------------------------------

void ClearON()
{
  ResetClearAll();
}

// -----------------------------------------------------------------------------------------------------------------

#if useWDT

ISR(WDT_vect, ISR_NAKED)                                     // Routine to deal with WDT overflow error 
{   

  cli();                                                      // disable interrupts

  ErrorFlag = true;                                          // Debugging message to user
  ErrorCode = 2; 

  MCUSR = 0;
  WDTCSR &= ~(1<<WDIF);                                      // Clear WDT Int. Flag
  mode = 4;

  sei();
  reti();
}


void WDT_Init()
{
  cli();                                                     //disable interrupts

  wdt_reset();                                               //reset watchdog


  WDTCSR = (1<<WDCE)|(1<<WDE);                               //set up WDT interrupt
  //WDTCSR = (1<<WDIE)|(1<<WDP2)|(1<<WDP1)|(1<<WDP0);                // 2 seconds prescaller ISR only ? 
  WDTCSR = (1<<WDIE)|(1<<WDE)|(1<<WDP2)|(1<<WDP1)|(1<<WDP0);       // 2 seconds prescaller ISR only and Reset
  //WDTCSR = (1<<WDE)|(1<<WDP2)|(1<<WDP1)|(1<<WDP0);                 // 2 seconds ISR only ? - > does not work - resets but hangs??

  MCUSR = 0;
  WDTCSR &= ~(1<<WDE);                                            // Turn off reset


  sei();                                                            //Enable global interrupts
}

#endif 

// -----------------------------------------------------------------------------------------------------------------

void playTone(long duration, int freq) {
// duration in mSecs, frequency in hertz
  PORTB = B00000000; //blank display
  duration *= 700;
  int period = (1.0 / freq) * 1000000;
  long elapsed_time = 0;
  int amax=0;
  byte up=1;

    while (elapsed_time < duration) {
      digitalWrite(speaker,HIGH);
      delayMicroseconds(period / 2);
      digitalWrite(speaker, LOW);
      delayMicroseconds(period / 2);
      elapsed_time += (period);  
  }  
}

// -----------------------------------------------------------------------------------------------------------------

void NumberToDigits(long TheNumber)
{
  tempLONG = 0;
  leadingDig = false;

  working = TheNumber;
  tempLONG = working/10000000L;   
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    leadingDig = true;
    MSDIndex = 7;
    MSD[8] = byte(tempLONG);
    working = working - (tempLONG*10000000L);
  }

  tempLONG = working/1000000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0 || leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 6;
    }
    leadingDig = true;
    //    disp[4] = byte(tempLONG);
    MSD[7] = byte(tempLONG);
    working = working - (tempLONG*1000000L);
  }

  tempLONG = working/100000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0 || leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 5;
    }
    leadingDig = true;
    //    disp[4] = byte(tempLONG);
    MSD[6] = byte(tempLONG);
    working = working - (tempLONG*100000L);
  }

  tempLONG = working/10000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0 || leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 4;
    }
    leadingDig = true;
    //    disp[4] = byte(tempLONG);
    MSD[5] = byte(tempLONG);
    working = working - (tempLONG*10000L);
  }

  tempLONG = working/1000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0 || leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 3;
    }
    leadingDig = true;
    //    disp[3] = byte(tempLONG);
    MSD[4] = byte(tempLONG); 
    working = working - (tempLONG*1000L);
  } 

  tempLONG = working/100L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0|| leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 2;
    }
    leadingDig = true;
    //    disp[2] = byte(tempLONG);
    MSD[3] = byte(tempLONG);
    working = working - (tempLONG*100L);
  }

  tempLONG = working/10L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0 || leadingDig) {
    if(!leadingDig)
    {
      MSDIndex = 1;
    }
    leadingDig = true;
    //    disp[1] = byte(tempLONG);
    MSD[2] = byte(tempLONG);
    working = working - (tempLONG*10L);
  }

  if(!leadingDig)
  {
    MSDIndex = 0;
  }

  //  disp[0] = byte(working);
  MSD[1] = byte(working);
  tempLONG = 0;
}      

// -----------------------------------------------------------------------------------------------------------------
void sayNumber(int number)
{
#if useWDT
  WDT_Init();
#endif 
 
  if (talking==true)
{
  if (female==true)
{
     switch (number) {
    case 0: 
      talkieSay(spZERO);  
      break;
    case 1: 
      talkieSay(spONE2); 
      break;
    case 2: 
      talkieSay(spTWO2);  
      break;
    case 3: 
      talkieSay(spTHREE2); 
      break;
    case 4: 
      talkieSay(spFOUR2); 
      break;
    case 5: 
      talkieSay(spFIVE2);  
      break;
    case 6: 
      talkieSay(spSIX2);  
      break;    
    case 7: 
      talkieSay(spSEVEN2); 
      break;
    case 8: 
      talkieSay(spEIGHT2); 
      break;
    case 9: 
      talkieSay(spNINE2);  
      break;
    case 11: 
      talkieSay(spPOINT);  
      break;
    } 
}
else
{
     switch (number) {
    case 0: 
      talkieSay(spZERO);  
      break;
    case 1: 
      talkieSay(spONE); 
      break;
    case 2: 
      talkieSay(spTWO);  
      break;
    case 3: 
      talkieSay(spTHREE); 
      break;
    case 4: 
      talkieSay(spFOUR); 
      break;
    case 5: 
      talkieSay(spFIVE);  
      break;
    case 6: 
      talkieSay(spSIX);  
      break;    
    case 7: 
      talkieSay(spSEVEN); 
      break;
    case 8: 
      talkieSay(spEIGHT); 
      break;
    case 9: 
      talkieSay(spNINE);  
      break;
    case 11: 
      talkieSay(spPOINT);  
      break;
    } 
  }
  
  
  
  
  }
}


void DPToDigits(long TheNumber)
{
  tempLONG = 0;

  LSDIndex = 0;

  for(i = 1; i <7; i++)
  {
    LSD[i] = 0;
  }

  working = TheNumber;

  tempLONG = working/100000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    LSDIndex = 1;
    LSD[1] = byte(tempLONG);
    working = working - (tempLONG*100000L);
  }

  tempLONG = working/10000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    LSDIndex = 2;
    LSD[2] = byte(tempLONG);
    working = working - (tempLONG*10000L);
  }

  tempLONG = working/1000L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    LSDIndex = 3;
    LSD[3] = byte(tempLONG);
    working = working - (tempLONG*1000L);
  }

  tempLONG = working/100L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    LSDIndex = 4;
    LSD[4] = byte(tempLONG); 
    working = working - (tempLONG*100L);
  } 

  tempLONG = working/10L;
  tempLONG =  abs(tempLONG); 
  if(tempLONG > 0) {
    LSDIndex = 5;
    LSD[5] = byte(tempLONG);
    working = working - (tempLONG*10L);
  }

  if(working > 0) {
    LSDIndex = 6;
    LSD[6] = byte(working);
  }

  tempLONG = 0;
}

// -----------------------------------------------------------------------------------------------------------------

void FloatToDigits()
{

#if DoSerial 
  Serial.println("Start Disp update");
  Serial.println(firstNumber, 6);
  Serial.println(secondNumber, 6);
#endif

  if(firstNumber <0)
  {
    negativeFlag = true;

  }
  else
  {
    negativeFlag = false;
  }

  firstNumber = abs(firstNumber);
  //  tempFLOAT = firstNumber;

  tempLONG = 0;

  tempLONG = long(firstNumber);

  NumberToDigits(tempLONG);                                   // Fill MSD[] array with left side of result

  //  tempFLOAT = (firstNumber - long(firstNumber));              // Remove numbers to left of decimal

  //  tempFLOAT = tempFLOAT *10000.0;                             // Should shift decimal value to left

  firstNumber = (firstNumber - long(firstNumber));              // Remove numbers to left of decimal

  firstNumber = firstNumber *1000000.0;                             // Should shift decimal value to left  

  //  if((tempFLOAT - long(tempFLOAT)) > 0.5)                     // To Help with Arduino FLOAT rounding errors
  if((firstNumber - long(firstNumber)) > 0.5)                     // To Help with Arduino FLOAT rounding errors
  {
    //    tempLONG = long(tempFLOAT)  + 1L;                         // Remove any extra decimals and round up    
    tempLONG = long(firstNumber)  + 1L;                         // Remove any extra decimals and round up
  }
  else
  {
    //    tempLONG = long(tempFLOAT);                               // Remove any extra decimals    
    tempLONG = long(firstNumber);                               // Remove any extra decimals
  }

  if(tempLONG > 0)                                              // Jan 17th 2013
  {
    DPToDigits(tempLONG);                                       // Fill LSD[] array with right side of result
  }
  else
  {
    LSDIndex = 0;
  }

  for(i = 0; i<8;i++)                                         // Clear the array before reloading it
  {
    disp[i] = 10;
  }

  tempLONG = 0;                                          // Jan 20th
  firstNumber = 0.0;                                     // Jan 20th

  working = 1L;
  // tempLONG = long(MSD[1]);
  for(i = 0; i<=MSDIndex;i++)                            // Start filling display with whole number part of result
  {
    disp[i] = MSD[i+1]; 
    // tempLONG = long(MSD[i+1]) + (tempLONG*(i*10L));
    // tempLONG =  tempLONG + (long(MSD[i+1])*(i*10L)); < - broken - not to the power of ten

    tempLONG =  tempLONG + (long(MSD[i+1])*working);
    working = working * 10L;

#if DoSerial
    Serial.println(disp[i]);
#endif 
  }

  DPDEX = 0;                                             // Place decimal point after integer part of number

#if DoSerial
  Serial.println("LSDIndex");
  Serial.println(LSDIndex);
  Serial.println("MSDIndex");
  Serial.println(MSDIndex);
#endif

  if(LSDIndex>0)                                         // Check if there is a decimal part of the result
  {
    // --    
    tempINT = 8;                                         // Calculate display digits remaining for decimal part
    if(negativeFlag)                                     // Loose one digit to "-" symbol
    {
      tempINT = tempINT -1;
    }  
    tempINT = tempINT - (MSDIndex +1);                   // decimal value can only fill remaining digits

    if(tempINT >0)
    {
      // --    
      // if(tempINT <4 && LSDIndex > tempINT)
      // {
      //   LSDIndex = LSDIndex - (4-tempINT);
      // }

      LSDIndex = min(LSDIndex,tempINT);                  // Fill decimal place with lessor of remaining space or DP digits

      for(i = 1; i<=LSDIndex;i++)
      {
        //        ShiftLeft();

        //       ShiftLeft - S
        for(int i = 7; i >0;i--)
        {
          disp[i] = disp[i-1];
        }
        //        disp[0] = 0;                                   // Not required, we assign "0" value see lines below
        DPDEX = DPDEX +1;
        //       ShiftLeft - E

        disp[0] = LSD[i];

        firstNumber =(float(LSD[i])/pow(10.0,float(i))) + firstNumber;      // Jan 20th
      }
    }
  }
  firstNumber = float(tempLONG) + firstNumber;
  tempLONG = 0;

#if DoSerial  
  Serial.println("END Disp update");
  Serial.println(firstNumber, 6);
  Serial.println(secondNumber, 6);
#endif    
}

// -----------------------------------
